//
//  YHResignDocumentDetailViewController.swift
//  galaxy
//
//  Created by alexzzw on 2024/9/13.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
import SafariServices
import AttributedString

class YHResignDocumentDetailViewController: YHBaseViewController {
    
    private var editEvent: (() -> Void)?
    private var submitEvent: (() -> Void)?
    private var signatureConfirmationEvent: (() -> Void)?
    private var previewEvent: (() -> Void)?
    private var infoConfirmationEvent: (() -> Void)?

    enum RowType {
        case tips(_ title: String, _ detail: ASAttributedString)
        case content(_ title: String, _ fileName: String, _ status: YHResignDocumentStatus, _ config: YHResignDocumentStatusCell.ButtonsConfig)
    }
    
    private var datas: [[RowType]] = []
    private lazy var viewModel = YHResignDocumentViewModel()
    
    private let docId: Int
    private var name: String
    
    init(_ docId: Int, _ name: String = "") {
        self.docId = docId
        self.name = name
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    private lazy var tableView: UITableView = {
        let view = UITableView(frame:.zero, style:.grouped)
        view.estimatedSectionHeaderHeight = 16.0
        view.estimatedSectionFooterHeight = 0.01
        view.sectionHeaderHeight = 16.0
        view.sectionFooterHeight = 0.01
        view.contentInsetAdjustmentBehavior = .never
        view.showsVerticalScrollIndicator = false
        view.separatorStyle = .none
        view.delegate = self
        view.dataSource = self
        view.backgroundColor = .clear
        view.tableFooterView = UITableViewHeaderFooterView()
        view.rowHeight = UITableView.automaticDimension
        view.register(YHResignDocumentTipsCell.self, forCellReuseIdentifier: YHResignDocumentTipsCell.cellReuseIdentifier)
        view.register(YHResignDocumentStatusCell.self, forCellReuseIdentifier: YHResignDocumentStatusCell.cellReuseIdentifier)
        return view
    }()

    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
        tableView.reloadData()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        requestData(isNeedLoading: true)
    }
}

extension YHResignDocumentDetailViewController {
    private func setupUI() {
        gk_navTitle = name
        gk_navBarAlpha = 1.0
        gk_navBackgroundColor = .white
        
        view.backgroundColor = UIColor.contentBkgColor
        view.addSubview(tableView)
        
        tableView.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalTo(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalToSuperview()
        }
    }
    
    private func requestData(isNeedLoading: Bool = false) {
        if isNeedLoading {
            YHHUD.show(.progress(message: "加载中..."))
        }
        viewModel.getDocumentDetail(docId: docId) { [weak self] documentModel, error in
            if isNeedLoading {
                YHHUD.hide()
            }
            guard let documentModel = documentModel else {
                printLog("YHResignDocumentDetailViewController:  请求失败")
                if let errorMsg = error?.errorMsg, errorMsg.count > 0 {
                    YHHUD.flash(message: errorMsg)
                }
                return
            }
            guard let self = self else {
                return
            }
            var firstSetcion: [RowType] = []
            var secondSetcion: [RowType] = []
            if let docType = YHResignDocumentType(rawValue: documentModel.docType), let status = YHResignDocumentStatus.getStatus(documentModel.docStatus) {
                firstSetcion.append(.tips("​填写指引", docType.attributedTips()))
                let config = self.getStatusButtonsConfig(docType: docType, status: status, model: documentModel)
                secondSetcion = [.content("文书稿件", self.name, status, config)]
                datas.removeAll()
                datas.append(firstSetcion)
                datas.append(secondSetcion)
                tableView.reloadData()
            }
        }
    }
}

extension YHResignDocumentDetailViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        datas.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        guard datas.count > section else {
            return 0
        }
        return datas[section].count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard datas.count > indexPath.section else {
            return UITableViewCell()
        }
        let sectionArr = datas[indexPath.section]
        guard sectionArr.count > indexPath.row else {
            return UITableViewCell()
        }
        let item = sectionArr[indexPath.row]
        
        switch item {
        case let .tips(title, detail):
            if let cell = tableView.dequeueReusableCell(withIdentifier: YHResignDocumentTipsCell.cellReuseIdentifier) as? YHResignDocumentTipsCell {
                cell.setupCellASAttributedInfo(title: title, detail: detail)
                return cell
            }
        case let .content(title, fileName, status, config):
            if let cell = tableView.dequeueReusableCell(withIdentifier: YHResignDocumentStatusCell.cellReuseIdentifier) as? YHResignDocumentStatusCell {
                cell.setupCellInfo(title: title, fileName: fileName, status: status, buttonsConfig: config)
                return cell
            }
        }
        return UITableViewCell()
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        return UIView()
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        return UIView()
    }
}

extension YHResignDocumentDetailViewController {
    private func getStatusButtonsConfig(docType: YHResignDocumentType, status: YHResignDocumentStatus,  model: YHResignDocumentCompleteModel) -> YHResignDocumentStatusCell.ButtonsConfig {
        var config = YHResignDocumentStatusCell.ButtonsConfig()
        var actions: [(() -> Void)] = []
        editEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.getResginDocumentUrl(docId: model.docId, isEditable: true) { [weak self] model, error in
                YHHUD.hide()
                guard let self = self else {
                    return
                }
                if let urlString = model?.url, let url = URL(string: urlString) {
                    let safariViewController = SFSafariViewController(url: url)
                    safariViewController.dismissButtonStyle = .close
                    safariViewController.modalPresentationStyle = .fullScreen
                    self.present(safariViewController, animated: true, completion: nil)
                } else if let error = error {
                    let errorMsg = error.errorMsg
                    YHHUD.flash(message: errorMsg)
                }
            }
            printLog("###editEvent")
        }
        
        submitEvent = { [weak self] in
            guard let self = self else {
                return
            }
            // 产品说增加二次弹窗确认
            YHCommonAlertView.show("确认提交？", "", "取消", "确认", fullGuestureEnable: false) {
            } callBack: { [weak self] in
                guard let self = self else {
                    return
                }
                YHHUD.show(.progress(message: "加载中..."))
                self.viewModel.submitResginDocument(model.docId, nil) { success, error in
                    YHHUD.hide()
                    if success {
                        YHHUD.flash(message: "提交成功")
                        self.requestData(isNeedLoading: true)
                    } else if let error = error {
                        let errorMsg = error.errorMsg
                        YHHUD.flash(message: errorMsg)
                    }
                }
            }
            
            printLog("###submitEvent")
        }
        
        signatureConfirmationEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.resginDocumentSignature(model.docId, nil) { [weak self] signatureModel, error in
                YHHUD.hide()
                guard let self = self else {
                    return
                }
                guard let signatureModel = signatureModel else {
                    if let error = error {
                        printLog("###续期文书签字: \(error.errorMsg)")
                        let errorMsg = error.errorMsg
                        YHHUD.flash(message: errorMsg)
                    }
                    return
                }
                let url = signatureModel.url
                // 跳签字
                let vc = YHSignatureWebViewController()
                vc.titleName = self.name
                vc.realNameUrl = url
                self.navigationController?.pushViewController(vc)
            }
        }
        
        previewEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.previewResginDocument(docId: model.docId) { [weak self] model, error in
                YHHUD.hide()
                guard let self = self else {
                    return
                }
                if let urlString = model?.url, let url = URL(string: urlString) {
                    let safariViewController = SFSafariViewController(url: url)
                    safariViewController.dismissButtonStyle = .close
                    safariViewController.modalPresentationStyle = .fullScreen
                    self.present(safariViewController, animated: true, completion: nil)
                } else if let error = error {
                    let errorMsg = error.errorMsg
                    YHHUD.flash(message: errorMsg)
                }
            }
            printLog("###previewEvent")
        }
        
        infoConfirmationEvent = { [weak self] in
            guard let self = self else {
                return
            }
            
            printLog("跳转QMAS")
            if let param = YHLoginManager.shared.userModel?.token,param.count>5 {
                printLog(param)
                var url = YHBaseUrlManager.shared.curH5URL() + "superAppBridge.html#/guide/qmas"
                //https://test-hkdiy-h5.galaxy-immi.com/superAppBridge.html#/guide/qmas?token=xx&doc_id=xx
                url = url + "?param=" + YHLoginManager.shared.h5Token + "&doc_id=\(model.docId)"
                let vc = YHH5WebViewVC()
                vc.isFullScreenFlag = false
                vc.navTitle = "QMAS文件信息确认" //self.name
                vc.url = url
                self.navigationController?.pushViewController(vc)
            }
        }
        // 产品定的没有生成签证文书就隐藏签字确认按钮
        let isSignFileOK = model.signFileOrigin.count > 0
        
        switch docType {
        case .powerOfAttorney, .settlementPlan, .explanatoryStatement:
            switch status {
            case .pendingConfirmation, .awaitingSignature:
                if let editEvent = editEvent {
                    actions.append(editEvent)
                }
                if let submitEvent = submitEvent {
                    actions.append(submitEvent)
                }
                
                if isSignFileOK {
                    config.names = ["在线编辑", "提交", "签字确认"]
                    config.style = .three
                    if let signatureConfirmationEvent = signatureConfirmationEvent {
                        actions.append(signatureConfirmationEvent)
                    }
                } else {
                    config.names = ["在线编辑", "提交"]
                    config.style = .two
                }
                config.actions = actions
                
            case .finalizing, .underReview, .completed:
                if let previewEvent = previewEvent {
                    actions.append(previewEvent)
                }
                config.names = ["查看"]
                config.actions = actions
                config.style = .one
            }
        case .qmasDoc:
            switch status {
            case .pendingConfirmation:
                if let infoConfirmationEvent = infoConfirmationEvent {
                    actions.append(infoConfirmationEvent)
                }
                config.names = ["信息确认"]
                config.actions = actions
                config.style = .one
            case .finalizing, .underReview, .completed:
                if let previewEvent = previewEvent {
                    actions.append(previewEvent)
                }
                config.names = ["查看"]
                config.actions = actions
                config.style = .one
            case .awaitingSignature:
                if let infoConfirmationEvent = infoConfirmationEvent {
                    actions.append(infoConfirmationEvent)
                }
                if isSignFileOK {
                    config.names = ["信息确认", "签字确认"]
                    config.style = .two
                    if let signatureConfirmationEvent = signatureConfirmationEvent {
                        actions.append(signatureConfirmationEvent)
                    }
                } else {
                    config.names = ["信息确认"]
                    config.style = .one
                }
                
                config.actions = actions
                
            }
        }
        return config
    }
}

private extension YHResignDocumentType {
    
    func attributedTips() -> ASAttributedString {
        let dotStr: ASAttributedString = "\(.image(UIImage(named: "tip_list_index_dot") ?? UIImage(), .custom(.center, size: .init(width: 3, height: 3))))"
        let spaceStr: ASAttributedString = "\(.image(UIImage(), .custom(.center, size: .init(width: 8, height: 3))))"
        let indentStr = dotStr + spaceStr
        switch self {
        case .powerOfAttorney:
            let firstLine: ASAttributedString = .init(string: "请您尽快核对信息，如信息有误，可进行在线编辑\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let secondLineL: ASAttributedString = .init(string: "如信息无误，请主申请人在", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let secondLineR: ASAttributedString = .init(string: "右下角的位置完成电子签字", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.brandMainColor))
            let secondLine = (secondLineL + secondLineR)
            return (indentStr + firstLine + indentStr + secondLine).add(attributes: .paragraph(.lineSpacing(8), .firstLineHeadIndent(0), .headIndent(11)))
        case .qmasDoc:
            let firstLine1: ASAttributedString = .init(string: "需填写完整邮箱、电话号码", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor))
            let firstLine2: ASAttributedString = .init(string: "（内地电话号码或香港电话号码）", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor50))
            let firstLine3: ASAttributedString = .init(string: "、日期", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor))
            let firstLine4: ASAttributedString = .init(string: "（填写当下日期即可）\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor50))
            let secondLine: ASAttributedString = .init(string: "请主申请人勾选信息并完成电子签字", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.brandMainColor))
            return (indentStr + firstLine1 + firstLine2 + firstLine3 + firstLine4 + indentStr + secondLine).add(attributes: .paragraph(.lineSpacing(8), .firstLineHeadIndent(0), .headIndent(11)))
        case .settlementPlan:
            let firstLine: ASAttributedString = .init(string: "如需调整和补充内容，请在文档中进行编辑\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let secondLine: ASAttributedString = .init(string: "编辑完成后，点击“完成编辑，提交”\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let third: ASAttributedString = .init(string: "我们将根据您的修改调整文书\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let fourth1: ASAttributedString = .init(string: "如您对《定居计划书》无其他补充建议，", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let fourth2: ASAttributedString = .init(string: "请您在文书尾部进行电子签字", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.brandMainColor))
            return (indentStr + firstLine + indentStr + secondLine + indentStr + third + indentStr + fourth1 + fourth2).add(attributes: .paragraph(.lineSpacing(8), .firstLineHeadIndent(0), .headIndent(11)))
        case .explanatoryStatement:
            let firstLine: ASAttributedString = .init(string: "如需调整和补充内容，请在文档中进行编辑\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let secondLine: ASAttributedString = .init(string: "编辑完成后，点击“完成编辑，提交”\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let third: ASAttributedString = .init(string: "我们将根据您的修改调整文书\n", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let fourth1: ASAttributedString = .init(string: "如您对《解释说明书》无其他补充建议，", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.mainTextColor70))
            let fourth2: ASAttributedString = .init(string: "请您在文书尾部进行电子签字", .font(UIFont.PFSC_R(ofSize: 12)), .foreground(UIColor.brandMainColor))
            return (indentStr + firstLine + indentStr + secondLine + indentStr + third + indentStr + fourth1 + fourth2).add(attributes: .paragraph(.lineSpacing(8), .firstLineHeadIndent(0), .headIndent(11)))
        }
    }
}
